#include "stdafx.h"
#include "effect.h"

void VDShaderTransformFunctionSimplify(VDShaderFunction& func) {
	VDShaderFunction::Insns insns;

	VDShaderFunction::Insns::const_iterator it(func.mInsns.begin()), itEnd(func.mInsns.end());
	for(; it != itEnd; ++it) {
		const VDShaderInsn& insn = *it;
		VDShaderInsnSource t0 = VDShaderInsnSource::reg(func.mRegisterCount + kVDShaderRegR0);
		VDShaderInsnSource t1 = VDShaderInsnSource::reg(func.mRegisterCount + kVDShaderRegR0 + 1);
		VDShaderInsnSource r0 = VDShaderInsnSource::reg(insn.mDestRegister);
		const VDShaderInsnSource& v0 = insn.mSources[0];
		const VDShaderInsnSource& v1 = insn.mSources[1];
		const VDShaderInsnSource& v2 = insn.mSources[2];

		switch(insn.mOp) {
			case kVDShaderOp_nop:
				break;

			case kVDShaderOp_crs:
				// crs r0, v0, v1 becomes:
				//	mul t0, v0.zxy, v1.yzx
				//	mad r0, v0.yzx, v1.zxy, -t0

				insns.push_back(VDShaderInsn::op(kVDShaderOp_mul, t0.mRegister, 7, insn.mSources[0].swizzle(0x2010), insn.mSources[1].swizzle(0x1200)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mad, insn.mDestRegister, insn.mWriteMask, insn.mSources[0].swizzle(0x1200), insn.mSources[1].swizzle(0x2010), -t0));
				++func.mRegisterCount;
				break;

			case kVDShaderOp_lrp:
				// lrp r0, v0, v1, v2 becomes:
				//	add t0, v1, -v2
				//	mad r0, t0, v0, v2
				insns.push_back(VDShaderInsn::op(kVDShaderOp_add, t0.mRegister, insn.mWriteMask, v1, -v2));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mad, insn.mDestRegister, insn.mWriteMask, v0, v2));
				++func.mRegisterCount;
				break;

			case kVDShaderOp_m3x2:
				// m3x2 r0, v0, c0 becomes:
				//	dp3 r0.x, v0, c0
				//	dp3 r0.y, v0, c1
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp3, insn.mDestRegister, 1, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 0)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp3, insn.mDestRegister, 2, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 1)));
				break;

			case kVDShaderOp_m3x3:
				// m3x3 r0, v0, c0 becomes:
				//	dp3 r0.x, v0, c0
				//	dp3 r0.y, v0, c1
				//	dp3 r0.z, v0, c2
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp3, insn.mDestRegister, 1, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 0)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp3, insn.mDestRegister, 2, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 1)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp3, insn.mDestRegister, 4, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 2)));
				break;

			case kVDShaderOp_m3x4:
				// m3x4 r0, v0, c0 becomes:
				//	dp3 r0.x, v0, c0
				//	dp3 r0.y, v0, c1
				//	dp3 r0.z, v0, c2
				//	dp3 r0.w, v0, c3
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp3, insn.mDestRegister, 1, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 0)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp3, insn.mDestRegister, 2, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 1)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp3, insn.mDestRegister, 4, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 2)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp3, insn.mDestRegister, 8, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 3)));
				break;

			case kVDShaderOp_m4x3:
				// m4x3 r0, v0, c0 becomes:
				//	dp4 r0.x, v0, c0
				//	dp4 r0.y, v0, c1
				//	dp4 r0.z, v0, c2
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp4, insn.mDestRegister, 1, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 0)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp4, insn.mDestRegister, 2, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 1)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp4, insn.mDestRegister, 4, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 2)));
				break;

			case kVDShaderOp_m4x4:
				// m4x4 r0, v0, c0 becomes:
				//	dp4 r0.x, v0, c0
				//	dp4 r0.y, v0, c1
				//	dp4 r0.z, v0, c2
				//	dp4 r0.w, v0, c3
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp4, insn.mDestRegister, 1, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 0)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp4, insn.mDestRegister, 2, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 1)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp4, insn.mDestRegister, 4, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 2)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp4, insn.mDestRegister, 8, v0, VDShaderInsnSource::reg(insn.mSources[1].mRegister + 3)));
				break;

			case kVDShaderOp_nrm:
				// nrm r0, v0 becomes:
				//
				//	dp3 t0.x, v0, v0
				//	rsq t0.x, t0.x
				//	mul r0, t0.x, v0
				insns.push_back(VDShaderInsn::op(kVDShaderOp_dp3, t0.mRegister, 1, v0, v0));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_rsq, t1.mRegister, 1, t0));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mul, insn.mDestRegister, insn.mWriteMask, t1.swizzle(0x0000), v0));
				func.mRegisterCount += 2;
				break;

			case kVDShaderOp_pow:
				// pow r0, v0, v1 becomes:
				//	log t0.x, v0
				//	mul t1.x, t0.x, v1
				//	exp r0, t1.x
				insns.push_back(VDShaderInsn::op(kVDShaderOp_log, t0.mRegister, 1, v0));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mul, t1.mRegister, 1, t0.swizzle(0x0000), v1));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_exp, insn.mDestRegister, insn.mWriteMask, t1.swizzle(0x0000)));
				func.mRegisterCount += 2;
				break;

			case kVDShaderOp_sincos:
				// sincos r0, v0, v1, v2
				// mul t0.z, v0.x, v0.x
				// mad t0.xy, t0.z, v1.xy, v1.wz 
				// mad t0.xy, t0.xy, t0.z, v2.xy 
				// mad t0.xy, t0.xy, t0.z, v2.wz
				// mul t0.x, t0.x, v0.x
				// mul t0.xy, t0.xy, t0.x
				// add t0.xy, t0.xy, t0.xy
				// add t0.x, -t0.x, v2.z
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mul, t0.mRegister, 4, v0.swizzle(0x0000), v0.swizzle(0x0000)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mad, t0.mRegister, 3, t0.swizzle(0x2222), v1, v1.swizzle(0x3222)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mad, t0.mRegister, 3, t0, t0.swizzle(0x2222), v2));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mad, t0.mRegister, 3, t0, t0.swizzle(0x2222), v2.swizzle(0x3222)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mul, t0.mRegister, 1, t0, v0));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mul, t0.mRegister, 3, t0, t0.swizzle(0x0000)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_add, t0.mRegister, 3, t0, t0));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_add, t0.mRegister, 1, -t0, v2.swizzle(0x2222)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mov, r0.mRegister, insn.mWriteMask, t0));
				++func.mRegisterCount;
				break;

			case kVDShaderOp_sub:
				// sub r0, v0, v1 becomes:
				//	add r0, v0, -v1
				insns.push_back(VDShaderInsn::op(kVDShaderOp_add, insn.mDestRegister, insn.mWriteMask, v0, -v1));
				break;

			case kVDShaderOp_texldb:
				// texldb is just reduced to texld
				insns.push_back(VDShaderInsn::op(kVDShaderOp_texld, insn.mDestRegister, insn.mWriteMask, v0, v1));
				break;

			case kVDShaderOp_texldp:
				// texldp r0, v0, s0 becomes:
				//	rcp t0.x, v0.w
				//	mul t1.xy, v0.xy, t0.x
				//	texld r0, t1.xy, s0
				insns.push_back(VDShaderInsn::op(kVDShaderOp_rcp, t0.mRegister, 1, v0.swizzle(0x3333)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_mul, t1.mRegister, 3, v0, t0.swizzle(0x0000)));
				insns.push_back(VDShaderInsn::op(kVDShaderOp_texld, insn.mDestRegister, insn.mWriteMask, t1, v1));
				func.mRegisterCount += 2;
				break;

			default:
				insns.push_back(insn);
				break;
		}
	}

	func.mInsns.swap(insns);
}
